/****************************************************************************
*
*	  Module Filename:  Realised_Molecule_Type.cpp
*	  Module Name:      Realised_Molecule_Type
*	  Description:      Molecule_Type that is realised as a Molecule in the Reactor
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#include "globldef.h"
#include "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//--------------  Reactor  Subsystem  -----------------------------------------------------------------------------
#include    "Realised_Molecule_Type.h"
#include    "Tracker.h"
//--------------------------------------------------------------------------------------------------------------

//--------------  Chemistry  Subsystem  ---------------------------------------------------------------------------
#include    "Molecule.h"
#include    "Molecule_Type.h"
//--------------------------------------------------------------------------------------------------------------

//--------------  Statistics  Subsystem  ----------------------------------------------------------------------------
#include    "Simulation_Statistics.h"
//--------------------------------------------------------------------------------------------------------------
void    Realised_Molecule_Type::Add_Molecule(double  Tracker_Probability, list<Tracker*>* Tracker_List_Ptr)

//  Add a Molecule of the Realised_Molecule_Type. 
//  If a Tracker_List is specified attach it and add this MolType to each of its Trackers. 
//  Otherwise create a Tracker with the specified probability

{
    Assert ((Tracker_List_Ptr == NULL) or (Tracker_Probability == 0.0), "Realised_Molecule_Type::Add_Molecule - Attempt to attach a Tracker List and a randomly generated Tracker");
//  Create a Molecule and associate it with the Realised_Molecule_Type.   

    CREATE_OBJECT(Molecule, Molecule* Mol__Ptr = new Molecule(this, Tracker_Probability));

    if (Tracker_List_Ptr not_eq NULL)
    THEN
        for (list<Tracker*>::iterator Iter = Tracker_List_Ptr->begin();
             Iter not_eq Tracker_List_Ptr->end();
             Iter++)
        BEGIN
            (*Iter)->Add_MolType_To_Path(Get_Molecule_Type_Ptr());
        END

        Mol__Ptr->Attach_Tracker_List(Tracker_List_Ptr);
    ENDIF
/*
//  ===============  TEST ONLY  ***********

    list<Tracker*>* TrackList_Ptr = Mol__Ptr->Get_Tracker_List_Ptr();
    if (TrackList_Ptr not_eq NULL)
    THEN
        for (list<Tracker*>::iterator Iter2 = TrackList_Ptr->begin();
                Iter2 not_eq TrackList_Ptr->end();
                Iter2++)
        BEGIN
            string  Path = string ("");
            (*Iter2)->Get_Path_As_String(Path);
            cout  << "Realised_Molecule_Type::Add_Molecule  -  Tracker Path = " << Path << endl;
        END
    END
//  ===============  */

    TList__Of_Molecules_Of_Type_Ptr->Add(Mol__Ptr);
}
//---------------------------------------------------------------------------------------------------------------

void    Realised_Molecule_Type::Add_Molecule(list<Tracker*>* Tracker_List_Ptr)

//  Add a Molecule of the Realised_Molecule_Type and attach the specified Tracker List
{
    double Tracker_Probability = 0.0;
    Add_Molecule(Tracker_Probability, Tracker_List_Ptr);
}
//---------------------------------------------------------------------------------------------------------------

void    Realised_Molecule_Type::Add_Molecule(double  Tracker_Probability)

//  Add a Molecule of the Realised_Molecule_Type. Create a Tracker with the specified probability

{
    list<Tracker*>* Tracker_List_Ptr = NULL;
    Add_Molecule(Tracker_Probability, Tracker_List_Ptr);
}
//---------------------------------------------------------------------------------------------------------------
void    Realised_Molecule_Type::Add_Molecule()

//  //  Add a Molecule of the Realised_Molecule_Type. Don't create a Tracker

{
    double Tracker_Probability = 0.0;
    list<Tracker*>* Tracker_List_Ptr = NULL;
    Add_Molecule(Tracker_Probability, Tracker_List_Ptr);
}
//-----------------------------------------------------------------------------------------------------------------
bool Realised_Molecule_Type::Compare_MolType_Id(const Realised_Molecule_Type* x, const Realised_Molecule_Type* y)

//  Compare the identifiers of Realised_Molecule_Types. Return true if time of x less than y, otherwise false

{
    return x->Get_Molecule_Type_Ptr()->Get_Identifier() < y->Get_Molecule_Type_Ptr()->Get_Identifier();
}
//----------------------------------------------------------------------------------------------------------
Molecule_Type*  Realised_Molecule_Type::Get_Molecule_Type_Ptr() const

//  Return a pointer to the Realised_Molecule_Type's Molecule_Type

{
    return(Molecule_Type__Ptr);
}
//----------------------------------------------------------------------------------------------------------
int     Realised_Molecule_Type::Get_Num_Of_Mols_In_Reactor()

//      Returns the current Num_Of_Mols_In_Reactor of the
//      Realised_Molecule_Type in the Reactor

{
    return(TList__Of_Molecules_Of_Type_Ptr->Count());
}
//----------------------------------------------------------------------------------------------------------
int     Realised_Molecule_Type::Get_Num_Of_R_Int_Types_In_Reactor() const

//  Return the number of Realisable Interaction Types for this Realised_Molecule_Type

{
    return List__Of_Realisable_Int_Ptrs.items();
}
//-----------------------------------------------------------------------------------------------------------------

Indexed_Series* Realised_Molecule_Type::Get_Series_Ptr()

//  Return a pointer to the Realised_Molecule_Type's Indexed Series

{
    return  Series_Ptr;
}
//-----------------------------------------------------------------------------------------------------------------
Realised_Molecule_Type::Realised_Molecule_Type(Molecule_Type*  MolType_Ptr,
                                              const string& Indexed_Series_Name,
                                              const string& Index_Name, 
                                              const string& Value_Name,
                                              const string& Series_List)

//  Constructor: Create List of related objects and a Data Series
{
    CREATE_OBJECT(VCSList,TList__Of_Molecules_Of_Type_Ptr = new VCSList);
    
//  Link the Realised_Molecule_Type to its Molecule_Type

    Molecule_Type__Ptr = MolType_Ptr;

//  Create a Data Series for the Realised_Molecule_Type    
    
    Series_Ptr = Simulation_Statistics::Get_Ptr()->Create_Indexed_Series
                                                    (Indexed_Series_Name,
                                                     Index_Name, 
                                                     Value_Name,
                                                     Series_List,
                                                     true);
}
//----------------------------------------------------------------------------------------------------------------
Realised_Molecule_Type::~Realised_Molecule_Type()

{
    Simulation_Statistics::Get_Ptr()->Delete_Series(Series_Ptr);
    DELETE_OBJECT(VCSList, TList__Of_Molecules_Of_Type_Ptr);
}
//----------------------------------------------------------------------------------------------------------
DN::List<Realisable_Interaction_Type*>&  Realised_Molecule_Type::Get_Int_Type_List_Ref()

//  Return a refence to the Realised_Molecule_Type's  Relisable Interaction Types list
//  *******  PREFERABLE TO REPLACE TIS AS IT ALLOWS ACCESS TO PRIVATE DATA

{
    return List__Of_Realisable_Int_Ptrs;
}  
//-----------------------------------------------------------------------------------------------------------------
           
list<Tracker*>*   Realised_Molecule_Type::Remove_Molecule()

//  Remove one of the Realised_Molecule_Type's Molecules (choose the last)

//  Return a pointer to the list of attached Trackers

{
//    LOG_ENTRY(Event_Remove_Molecule::Log_Event(Reactor::Get_Reactor__Ptr(), static_cast<Molecule*> (TList__Of_Molecules_Of_Type_Ptr->Items[Get_Num_Of_Mols_In_Reactor() - 1])));
    Molecule*  Mol_To_Remove_Ptr  = static_cast<Molecule*> (TList__Of_Molecules_Of_Type_Ptr->Get_Item(Get_Num_Of_Mols_In_Reactor() - 1));

//  Get a pointer to any Tracker that may be attached to the Molecule

    list<Tracker*>*  Attached_Tracker_List_Ptr = Mol_To_Remove_Ptr->Get_Tracker_List_Ptr();

    DELETE_OBJECT(Molecule,Mol_To_Remove_Ptr);

    TList__Of_Molecules_Of_Type_Ptr->Remove_Last();

    return Attached_Tracker_List_Ptr;
}   
//-----------------------------------------------------------------------------------------------------------------






