/****************************************************************************
*
*	  Module Filename:  Cycle_Type.cpp
*	  Module Name:      Cycle_Type
*	  Description:      Recognise and Handle Cycle Types
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/


#include    "globldef.h"
#include    "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//--------------  Tracker  Subsystem  -----------------------------------------------------------------------------

#include  "Cycle_Type.h"
#include  "Tracker.h"

//---------------Chemistry Subsystem ----------------------------------------------------------------------------

#include  "Molecule_Type.h"

//===================================================================================================

void Cycle_Type::Identify_Cycle_Type_And_Length (Tracker*  Tracker_Ptr, 
                                                    int Closing_MolType_Id, 
                                                    string& Cycle_Type_Identifier,
                                                    int&  Cycle_Length)

//  Identify the Cycle Type and Length for the Tracker that has been passed as part of the Tracker Path

//  Any steps in the Tracker path before the cycle starts must be removed.
//  Then identify the cycle; the first part of teh Cycke Identifier is the Cycle Leader, and
//  is just the highest MolType Id in the cycle. This guarantees uniqueness of Cycle Identifiers.

{
    string Identifier_First_Part;  // The part of the Cycle_Type_Identifier that starts with the Cycle Leader
    string Identifier_Second_Part;
    bool Cycle_Start_Found = false;
    int Highest_MolType_Id_In_Cycle = INT_MIN;

    list<Molecule_Type*> Unadjusted_Cycle_Path;

//  First, extract the Cycle from the Tracker's path. There could be a non-cyclic part before 
//  the cycle starts.

    for (bool Continue_Flag = Tracker_Ptr->Iter_To_First_MolType();
         Continue_Flag == true;
         Continue_Flag = Tracker_Ptr->Iter_To_Next_MolType())
    BEGIN
        int Current_MolType_Id = Tracker_Ptr->Get_Current_MolType_Ptr()->Get_Identifier();  
        if (Cycle_Start_Found not_eq true)
        THEN
            if (Closing_MolType_Id == Current_MolType_Id)
            THEN
                Cycle_Start_Found = true;
            ENDIF
        ENDIF
        if (Cycle_Start_Found == true)
        THEN
            Unadjusted_Cycle_Path.push_back(Tracker_Ptr->Get_Current_MolType_Ptr());    
            Highest_MolType_Id_In_Cycle = max(Highest_MolType_Id_In_Cycle, Current_MolType_Id);
        ENDIF
    END

//  The non cyclic items in the path have now been removed, and so we can now calculate the 
//  cycle length that is to be passed back

    Cycle_Length = Unadjusted_Cycle_Path.size();

    Assert (Cycle_Start_Found == true, "Cycle_Manager::Identify_Cycle_Type - No cycle start found to match the Closing_MolType_Id");

//  Now determine the cycle identifier 

    bool Cycle_Leader_Found = false;
    int Cycle_Id_Leader = Highest_MolType_Id_In_Cycle;  // MolType Id that will appear first 
                                                        // in the Cycle Type Identifier

    list<Molecule_Type*>::iterator Iter;
    for (Iter = Unadjusted_Cycle_Path.begin();
         Iter not_eq Unadjusted_Cycle_Path.end();
         Iter++)
    BEGIN
        int Current_MolType_Id = (*Iter)->Get_Identifier();
        if (Cycle_Leader_Found == true)
        THEN
            Identifier_First_Part += To_String(Current_MolType_Id) + string(":");
        ELSE
            if (Current_MolType_Id == Cycle_Id_Leader)
            THEN
                Cycle_Leader_Found = true;
                Identifier_First_Part += To_String(Current_MolType_Id) + string(":");
            ELSE
                Identifier_Second_Part += To_String(Current_MolType_Id) + string(":");
            ENDIF
        ENDIF
    END 

    Assert (Cycle_Leader_Found == true, "Cycle_Manager::Identify_Cycle_Type - The Tracker that has been notified as a Cycle has an invalid highest Molecule Type Id so that the Cycle Leader cannot be identified.");

    Cycle_Type_Identifier = Identifier_First_Part + Identifier_Second_Part;
}
//---------------------------------------------------------------------------------------------------------------
